/*
 *  Copyright (C) 2000, 2001, 2002 Marco Pesenti Gritti
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "eel-gconf-extensions.h"
#include "galeon-embed-utils.h"
#include "galeon-embed-shell.h"
#include "prefs-strings.h"
#include "gul-bonobo-extensions.h"
#include "gul-gui.h"
#include "gul-download.h"

#include <string.h>
#include <bonobo/bonobo-i18n.h>
#include <bonobo/bonobo-ui-util.h>
#include <gtk/gtkdialog.h>
#include <gtk/gtkmessagedialog.h>
#include <libgnomevfs/gnome-vfs-uri.h>

/* FIXME show a message about saved file in complete callback ?*/

/**
 * galeon_embed_utils_save:
 * @window: the referrer window. Used to parent the dialogs.
 * @default_dir_pref: the gconf path to persist the directory selected by the user.
 * @ask_dest: ask the user the destination path
 * @ask_content: show the user an option to save the content 
 * of the web page (images, javascript...)
 * @persist: the #GaleonEmbedPersist referring to the url
 *
 * Download a save an url asking a location to the user when requested
 **/
void 
galeon_embed_utils_save (GtkWidget *window,
		    	 const char *default_dir_pref,
		    	 gboolean ask_dest,
		    	 gboolean ask_content,
			 gboolean update_state,
		    	 GaleonEmbedPersist *persist)
{
	GnomeVFSURI *uri;
        char *retPath = NULL;
        char *fileName = NULL;
        char *dirName = NULL;
        char *retDir;
	char *target;
	const char *source;
        gresult ret;
	GaleonEmbed *embed;
	EmbedPersistFlags flags;
	gboolean content;

	g_object_ref (G_OBJECT(persist));
	
	galeon_embed_persist_get_flags (persist, &flags);
	
	galeon_embed_persist_get_source (persist, &source);

	if (source)
	{
		target = g_strdup (source);
	}
	else
	{
		galeon_embed_persist_get_embed (persist, &embed);
		g_return_if_fail (embed != NULL);
		
		galeon_embed_get_location (embed,
					   flags & 
					   EMBED_PERSIST_MAINDOC,
					   FALSE,
 				  	   &target);
	}

        /* Get a filename from the target url */
        uri = gnome_vfs_uri_new (target);
        if (uri)
        {
                fileName = gnome_vfs_uri_extract_short_name (uri);
                gnome_vfs_uri_unref (uri);
        }

        dirName = eel_gconf_get_string (default_dir_pref);
        if (dirName && dirName[0] == '\0')
        {
     		g_free (dirName);
                dirName = NULL;
        }

        if (!dirName || strcmp (dirName,"~") == 0)
        {
                g_free (dirName);
                dirName = g_strdup(g_get_home_dir ());
        }

	/* If we aren't asking for downloading dir, check that we aren't
	 * overwriting anything.  If we are, pop up a warning dialog and show
	 * the filepicker if the user doesn't want to overwrite the file.
	 */
	ret = G_FAILED;
	if (!ask_dest)
	{
		retPath = g_build_filename (dirName, fileName, NULL);
		if (gul_gui_confirm_overwrite_file (window, retPath))
		{
			ret = G_OK;
		}
		else
		{
			g_free (retPath);
			ask_dest = TRUE;
		}
	}

	if (ask_dest)
	{
        	/* show the file picker */
        	ret = galeon_embed_shell_show_file_picker 
					(embed_shell, window,
                                        _("Select the destination filename"),
                                        dirName, fileName, modeSave, &retPath,
                                        ask_content ? &content : NULL,
                                        NULL, NULL);
	}

        if (ret == G_OK)
        {
                uri = gnome_vfs_uri_new (retPath);
		g_return_if_fail (uri != NULL);
		
                retDir = gnome_vfs_uri_extract_dirname (uri);

                if (ask_content && content)
		{
			flags |= EMBED_PERSIST_SAVE_CONTENT;
			galeon_embed_persist_set_flags (persist,
							flags);
		}

		galeon_embed_persist_set_dest (persist, retPath);
			
		galeon_embed_persist_save (persist);
						
                /* set default save dir */
		if (update_state)
		{
                	eel_gconf_set_string (default_dir_pref, 
                        	             retDir);
		}
		
                g_free (retDir);
		g_free (retPath);
                gnome_vfs_uri_unref (uri);
        }
	
	g_object_unref (G_OBJECT(persist));
	
        g_free (dirName);
        g_free (fileName);
}

void galeon_embed_utils_save_event_property(GaleonEmbed *embed,
					    GaleonEmbedEvent *event,
					    gboolean ask_dest,
					    gboolean show_progress,
					    gboolean update_state,
					    const char *pref_dir,
					    const char *pref_state_dir,
					    const char *property)
{
	const char *location;
	const GValue *value;
	GtkWidget *window;
	GaleonEmbedPersist *persist;
	
	g_assert(IS_GALEON_EMBED_EVENT(event));
	g_assert(IS_GALEON_EMBED(embed));
	
	galeon_embed_event_get_property (event, property, &value);
	location = g_value_get_string (value);

	window = gtk_widget_get_toplevel (GTK_WIDGET(embed));
	g_return_if_fail(window != NULL);
	
	persist = galeon_embed_persist_new (embed);
	
	galeon_embed_persist_set_source (persist, location);

	if (show_progress)
	{
		galeon_embed_persist_set_flags (persist, 
						EMBED_PERSIST_SHOW_PROGRESS);
	}
	
	if (!ask_dest && pref_dir) 
	{
		/* if we're not asking for download directory, make sure the
		 * file is downloaded in the specified download directory and
		 * not somewhere else */
		char *dir = eel_gconf_get_string (pref_dir);
		eel_gconf_set_string (pref_state_dir, dir);
		g_free (dir);
	}

	galeon_embed_utils_save (window,
				 pref_state_dir,
				 ask_dest,
                                 FALSE, /* ask_content */
				 update_state,
                                 persist);

	g_object_unref (persist);
}

void galeon_embed_utils_download_event_property(GaleonEmbed *embed,
						GaleonEmbedEvent *event,
						gboolean ask_dest,
						gboolean update_state,
						const char *pref_dir,
						const char *pref_state_dir,
						const char *property)
{
       if (eel_gconf_get_boolean(CONF_DOWNLOADING_EXTERNAL_DOWNLOADER) ==
            DOWNLOADER_EXTERNAL)
        {
		const char *location;
		const GValue *value;

		galeon_embed_event_get_property (event, property, &value);
		location = g_value_get_string (value);

		return gul_download_external_save_url (location);
        }
	else
	{
		galeon_embed_utils_save_event_property (embed,event,
							eel_gconf_get_boolean 
				   			(CONF_DOWNLOADING_ASK_DIR),
				   			TRUE, /* show_progress */
							update_state,
							CONF_DOWNLOADING_DIR,
							CONF_STATE_LAST_DOWNLOAD_DIR,
							property);
	}
}

/**
 * galeon_embed_utils_handlernotfound_dialog_run:
 * @parent: the dialog parent window
 *
 * Show a dialog to warn the user that no application capable
 * to open the specified file are found. Used in the downloader
 * and in the mime type dialog.
 **/
void
galeon_embed_utils_nohandler_dialog_run (GtkWidget *parent)
{
        GtkWidget *dialog;
        
	/* FIXME mime db shortcut */
	
	dialog = gtk_message_dialog_new 
		(GTK_WINDOW(parent), 
                 GTK_DIALOG_MODAL,
                 GTK_MESSAGE_ERROR,
                 GTK_BUTTONS_OK,
                 _("No available applications to open "
                   "the specified file."));
        gtk_dialog_run (GTK_DIALOG(dialog));
	gtk_widget_destroy (dialog);
}

/**
 * galeon_embed_utils_free_page_properties
 * @props: Properties struct to free
 *
 * PageProperties is a complex struct who's members need freeing.
 * Here's a helper to do the job for you.
 **/
void
galeon_embed_utils_free_page_properties(EmbedPageProperties *props)
{
	/* FIXME: Will add fields as they get hooked up */

	g_free(props->content_type);
	g_free(props->encoding);
	g_free(props->referring_url);
	g_free(props->cipher_name);
	g_free(props->cert_issuer_name);

	if (props->metatags)
	{
		GList *i;
		for(i = props->metatags ; i ; i = i->next)
		{
			g_free(((EmbedPageMetaTag *)i->data)->name);
			g_free(((EmbedPageMetaTag *)i->data)->content);
			g_free(i->data);
		}
		g_list_free(props->metatags);
	}

	if (props->links)
	{
		GList *i;
		for(i = props->links ; i ; i = i->next)
		{
			g_free(((EmbedPageLink *)i->data)->url);
			g_free(((EmbedPageLink *)i->data)->title);
			g_free(((EmbedPageLink *)i->data)->rel);
			g_free(i->data);
		}
		g_list_free(props->links);
	}

	if (props->forms)
	{
		GList *i;
		for(i = props->forms ; i ; i = i->next)
		{
			g_free(((EmbedPageForm *)i->data)->name);
			g_free(((EmbedPageForm *)i->data)->method);
			g_free(((EmbedPageForm *)i->data)->action);
			g_free(i->data);
		}
		g_list_free(props->forms);
	}

	if (props->images)
	{
		GList *i;
		for(i = props->images ; i ; i = i->next)
		{
			g_free(((EmbedPageImage *)i->data)->url);
			g_free(((EmbedPageImage *)i->data)->alt);
			g_free(((EmbedPageImage *)i->data)->title);
			g_free(i->data);
		}
		g_list_free(props->images);
	}

	/* As these lists are properly defined, more
	 * complicated free procedures will be needed
	 */
	g_list_free(props->stylesheets);
	g_free(props);
}
