/*
 *  Copyright (C) 2003  Ricardo Fernndez Pascual
 *  Copyright (C) 2003  Philip Langdale
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <libgnome/gnome-i18n.h>
#include <gtk/gtkradiomenuitem.h>
#include <gtk/gtktearoffmenuitem.h>
#include <gtk/gtkmenu.h>
#include <gtk/gtkimage.h>
#include "galeon-window-menu.h"
#include "galeon-shell.h"
#include "session.h"
#include "gul-gobject-misc.h"
#include "galeon-marshal.h"
#include "gul-string.h"
#include "gul-gui.h"

//#define DEBUG_MSG(x) g_print x
#define DEBUG_MSG(x)

/**
 * Private data
 */
struct _GaleonWindowMenuPrivate {
	Session *session;
	GaleonWindow *window;
	GtkMenuShell *ms;
	guint rebuild_timeout;
	gboolean signals_registered;

	GSList *menuitems;
	const GList *windows;

	gint ignore_menu_callbacks;

	GHashTable *menuitem_to_window;

	GtkWidget *parent_widget;
};

#define REBUILD_TIMEOUT 700

/**
 * Private functions, only availble from this file
 */
static void		galeon_window_menu_class_init		(GaleonWindowMenuClass *klass);
static void		galeon_window_menu_init			(GaleonWindowMenu *e);
static void		galeon_window_menu_finalize_impl	(GObject *o);
static void		galeon_window_menu_build_submenu	(GaleonWindowMenu *gm);
static void		galeon_window_menu_build_item		(GaleonWindowMenu *gm,
								 GaleonWindow *window,
								 guint index);

static void		galeon_window_menu_rebuild		(GaleonWindowMenu *gm);
static void 		galeon_window_menu_rebuild_with_timeout	(GaleonWindowMenu *gm);
static gboolean		galeon_window_menu_rebuild_timeout_cb	(gpointer data);
static void		galeon_window_menu_activate_cb		(GtkMenuItem *mi,
								 GaleonWindowMenu *cm);
static void		galeon_window_menu_new_window_cb	(Session *session,
								 GaleonWindow* window,
								 GaleonWindowMenu *gm);
static void		galeon_window_menu_close_window_cb	(Session *session,
								 GaleonWindowMenu *gm);
static void		galeon_window_menu_embed_change_cb	(GaleonWindow *window,
								 GaleonEmbed *old_embed,
								 GaleonEmbed *new_embed,
								 GaleonWindowMenu *wm);
static void		galeon_window_menu_embed_title_cb	(GaleonEmbed *embed,
								 GaleonWindowMenu *wm);

static gpointer g_object_class;

/**
 * GtkMenu object
 */

MAKE_GET_TYPE (galeon_window_menu, "GaleonWindowMenu", GaleonWindowMenu, galeon_window_menu_class_init,
	       galeon_window_menu_init, G_TYPE_OBJECT);

static void
galeon_window_menu_class_init (GaleonWindowMenuClass *klass)
{
	G_OBJECT_CLASS (klass)->finalize = galeon_window_menu_finalize_impl;
	g_object_class = g_type_class_peek_parent (klass);

}

static void 
galeon_window_menu_init (GaleonWindowMenu *m)
{
	GaleonWindowMenuPrivate *p = g_new0 (GaleonWindowMenuPrivate, 1);
	m->priv = p;
	
	p->menuitem_to_window = g_hash_table_new (NULL, NULL);
	p->ignore_menu_callbacks = 0;
	p->parent_widget = NULL;
	p->signals_registered = FALSE;
}

static void
galeon_window_menu_finalize_impl (GObject *o)
{
	GaleonWindowMenu *wm = GALEON_WINDOW_MENU(o);
	GaleonWindowMenuPrivate *p = wm->priv;
	GSList *li;
	GList *i, *windows = NULL;

	DEBUG_MSG (("Finalizing GaleonWindowMenu\n"));

	if (p->session)
	{
		g_object_remove_weak_pointer(G_OBJECT(p->session),
					     (gpointer *)&p->session);

		g_signal_handlers_disconnect_matched(p->session,
						     G_SIGNAL_MATCH_DATA,
						     0, 0, NULL, NULL, wm);

		windows = session_get_windows(p->session);
	}

	for (i = windows; i; i = i->next)
	{
		GaleonWindow *window = GALEON_WINDOW(i->data);
		GaleonEmbed *embed = galeon_window_get_active_embed(window);

		g_signal_handlers_disconnect_matched(window,
						     G_SIGNAL_MATCH_DATA,
						     0, 0, NULL, NULL, wm);

		if (embed)
			g_signal_handlers_disconnect_matched(embed,
							     G_SIGNAL_MATCH_DATA,
							     0, 0, NULL, NULL, wm);
	}

	galeon_window_menu_set_parent_widget (wm, NULL);
	
	p->window = NULL;

	if (p->rebuild_timeout)
	{
		g_source_remove (p->rebuild_timeout);
	}

	for (li = p->menuitems; li; li = li->next)
	{
		g_object_unref (li->data);
	}
	g_slist_free (p->menuitems);

	p->windows = NULL;

	if (p->ms)
	{
		g_object_unref (p->ms);
	}
	g_hash_table_destroy (p->menuitem_to_window);

	g_free (p);

	G_OBJECT_CLASS (g_object_class)->finalize (o);
}

GaleonWindowMenu *
galeon_window_menu_new (GtkMenuShell *ms)
{
	GaleonWindowMenu *ret = g_object_new (GALEON_TYPE_WINDOW_MENU, NULL);
	GaleonWindowMenuPrivate *p = ret->priv;

	p->window = NULL;

	p->ms = g_object_ref (ms);

	p->menuitems = NULL;
	p->windows = NULL;

        p->session = galeon_shell_get_session(galeon_shell);
        if (p->session)
        {
                g_object_add_weak_pointer(G_OBJECT(p->session),
                                         (gpointer *)&p->session);
        }

	galeon_window_menu_rebuild_with_timeout (ret);

	return ret;
}

static void
galeon_window_menu_build_submenu (GaleonWindowMenu *gm)
{
	GaleonWindowMenuPrivate *p = gm->priv;
	const GList *li;
	guint i=0;

	p->ignore_menu_callbacks++;

	gul_gui_setup_tearoff (p->ms);

	p->windows = session_get_windows(p->session);

	if (p->windows)
	{
		for (li = p->windows; li; li = li->next, i++)
		{
			galeon_window_menu_build_item (gm, li->data, i);
		}
	}
	else
	{
		p->windows = NULL;
	}

	if (GTK_IS_WIDGET (p->parent_widget))
	{
		if (p->windows)
		{
			gtk_widget_set_sensitive (p->parent_widget, TRUE);
		}
		else
		{
			gtk_widget_set_sensitive (p->parent_widget, FALSE);
		}
	}
	p->ignore_menu_callbacks--;
}

#define MENU_ITEM_MAX_LENGTH 40
static void
galeon_window_menu_build_item (GaleonWindowMenu *gm,
			       GaleonWindow *window,
			       guint index)
{
	GaleonWindowMenuPrivate *p = gm->priv;
	GtkWidget *w;
	GaleonTab *tab = galeon_window_get_active_tab(window);
	gchar *title_s = gul_string_shorten(galeon_tab_get_title(tab),
					    MENU_ITEM_MAX_LENGTH);
	gchar *label = gul_string_new_num_accel (index, title_s, TRUE);

	p->ignore_menu_callbacks++;

	w = gtk_menu_item_new_with_mnemonic(label);

	g_signal_connect (w, "activate", 
			  G_CALLBACK (galeon_window_menu_activate_cb), gm);

	p->menuitems = g_slist_prepend (p->menuitems, g_object_ref (w));

	g_hash_table_insert (p->menuitem_to_window, w, window);

	gtk_widget_show (w);
	gtk_container_add (GTK_CONTAINER (p->ms), w);

	if (window == p->window)
		gtk_widget_set_sensitive(w, FALSE);

	g_free(title_s);
	g_free(label);

	p->ignore_menu_callbacks--;
}

static void
galeon_window_menu_activate_cb (GtkMenuItem *mi, GaleonWindowMenu *cm)
{
        GaleonWindow *window;
	GaleonWindowMenuPrivate *p = cm->priv;
	
	if (p->ignore_menu_callbacks > 0)
	{
		return;
	}

	DEBUG_MSG (("galeon_window_menu_activated_cb\n"));

	window = g_hash_table_lookup (p->menuitem_to_window, mi);
	if (window)
	{
		DEBUG_MSG (("Moving to window '%s'\n", galeon_tab_get_location(galeon_window_get_active_tab(window))));
	
		g_return_if_fail (IS_GALEON_WINDOW (p->window));
		galeon_window_reparent_tab(window, p->window,
					   galeon_window_get_active_tab(p->window));
	}
	else
	{
		DEBUG_MSG (("NULL item assoicated to menu item\n"));
	}
}

static gboolean
galeon_window_menu_rebuild_timeout_cb (gpointer data)
{
	GaleonWindowMenu *gm = data;
	GaleonWindowMenuPrivate *p = gm->priv;
	DEBUG_MSG (("GaleonWindowMenu rebuild timeout\n"));

	p->rebuild_timeout = 0;

	galeon_window_menu_rebuild (data);
	return FALSE;
}

static void 
galeon_window_menu_rebuild_with_timeout (GaleonWindowMenu *gm) 
{
	GaleonWindowMenuPrivate *p = gm->priv;

	DEBUG_MSG (("galeon_window_menu_rebuild_with_timeout\n"));

	if (p->rebuild_timeout)
	{
		g_source_remove (p->rebuild_timeout);
	}
	
	p->rebuild_timeout = g_timeout_add (REBUILD_TIMEOUT, 
					    galeon_window_menu_rebuild_timeout_cb, gm);
}

static void
galeon_window_menu_reset (GaleonWindowMenu *gm)
{
	GaleonWindowMenuPrivate *p = gm->priv;
	GList *children;
	GList *li;
	GSList *sli;

	p->ignore_menu_callbacks++;

	/* free the list of menuitems */
	for (sli = p->menuitems; sli; sli = sli->next)
	{
		g_signal_handlers_disconnect_matched (sli->data, G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, gm);
		g_object_unref (sli->data);
	}
	g_slist_free (p->menuitems);
	p->menuitems = NULL;

	p->windows = NULL;

	if (g_hash_table_size (p->menuitem_to_window) > 0)
	{
		g_hash_table_destroy (p->menuitem_to_window);
		p->menuitem_to_window = g_hash_table_new (NULL, NULL);
	}

	children = gtk_container_get_children (GTK_CONTAINER (p->ms));
	for (li = children; li; li = li->next)
	{
		if (!GTK_IS_TEAROFF_MENU_ITEM (li->data))
		{
			gtk_container_remove (GTK_CONTAINER (p->ms), li->data);
		}
	}
	g_list_free (children);

	p->ignore_menu_callbacks--;
}

static void
galeon_window_menu_rebuild (GaleonWindowMenu *gm)
{
	GaleonWindowMenuPrivate *p = gm->priv;

	DEBUG_MSG (("galeon_window_menu_rebuild called\n"));

	p->ignore_menu_callbacks++;

	galeon_window_menu_reset (gm);
	
	galeon_window_menu_build_submenu (gm);

	p->ignore_menu_callbacks--;
}

void 
galeon_window_menu_set_window(GaleonWindowMenu *wm, GaleonWindow *w)
{
	GaleonWindowMenuPrivate *p;
	GList *i, *windows;

	g_return_if_fail(GALEON_IS_WINDOW_MENU (wm));
	g_return_if_fail(w && IS_GALEON_WINDOW (w));

	DEBUG_MSG(("galeon_window_menu_set_window\n"));

	p = wm->priv;
	windows = session_get_windows(p->session);

	galeon_window_menu_reset(wm);

	p->window = w;

	if (p->signals_registered) return;

	for (i=windows; i ; i=i->next)
	{
		galeon_window_menu_new_window_cb(p->session,
						 GALEON_WINDOW(i->data),
						 wm);
	}

	g_signal_connect(p->session, "new_window",
			 G_CALLBACK(galeon_window_menu_new_window_cb), wm);
	g_signal_connect(p->session, "close_window",
			 G_CALLBACK(galeon_window_menu_close_window_cb), wm);

	p->signals_registered = TRUE;

	galeon_window_menu_rebuild_with_timeout(wm);
}

void
galeon_window_menu_set_parent_widget (GaleonWindowMenu *cm, GtkWidget *w)
{
	GaleonWindowMenuPrivate *p = cm->priv;
	
	if (p->parent_widget)
	{
		g_object_remove_weak_pointer (G_OBJECT (p->parent_widget),
					      (gpointer *) &p->parent_widget);
	}

	p->parent_widget = w;

	if (p->parent_widget)
	{
		g_object_add_weak_pointer (G_OBJECT (p->parent_widget), 
					   (gpointer *) &p->parent_widget);
	}
}

static void
galeon_window_menu_new_window_cb(Session *session,
				 GaleonWindow* window,
				 GaleonWindowMenu *wm)
{
	GaleonEmbed *embed = galeon_window_get_active_embed(window);

	g_signal_connect(window, "active_embed_changed",
			 G_CALLBACK(galeon_window_menu_embed_change_cb), wm);
	if (embed)
		g_signal_connect(embed, "ge_title",
				 G_CALLBACK(galeon_window_menu_embed_title_cb), wm);

	galeon_window_menu_rebuild_with_timeout(wm);
}

static void
galeon_window_menu_close_window_cb(Session *session,
				   GaleonWindowMenu *wm)
{
	GaleonWindow *window = wm->priv->window;
	GaleonEmbed *embed = galeon_window_get_active_embed(window);

	g_signal_handlers_disconnect_matched(window,
					     G_SIGNAL_MATCH_DATA,
					     0, 0, NULL, NULL, wm);

	if (embed)
		g_signal_handlers_disconnect_matched(embed,
						     G_SIGNAL_MATCH_DATA,
						     0, 0, NULL, NULL, wm);

	galeon_window_menu_rebuild_with_timeout(wm);
}

static void
galeon_window_menu_embed_change_cb(GaleonWindow *window,
				   GaleonEmbed *old_embed,
				   GaleonEmbed *new_embed,
				   GaleonWindowMenu *wm)
{
	if (old_embed)
		g_signal_handlers_disconnect_matched(old_embed,
						     G_SIGNAL_MATCH_DATA,
						     0, 0, NULL, NULL, wm);
	if (new_embed)
		g_signal_connect(new_embed, "ge_title",
				 G_CALLBACK(galeon_window_menu_embed_title_cb), wm);

	galeon_window_menu_rebuild_with_timeout(wm);
}

static void
galeon_window_menu_embed_title_cb(GaleonEmbed *embed,
				  GaleonWindowMenu *wm)
{
	galeon_window_menu_rebuild_with_timeout(wm);
}
